package com.gct.tools.etlcamelhuge.MQ;

import com.alibaba.fastjson.JSONObject;
import com.gct.tools.etlcamelhuge.entity.DiagnoseMsg;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.exception.MQClientException;
import org.apache.rocketmq.client.producer.DefaultMQProducer;
import org.apache.rocketmq.common.message.Message;

import java.nio.charset.StandardCharsets;

/**
 * class name: DiagnoseMessageProducer
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/6/24 下午6:44
 */
@Slf4j
public class DiagnoseMessageProducer implements MessageProducer {
    private final DefaultMQProducer producer;
    private final GCTRocketMQProperties properties;

    private final SendFailCallBack failCallBack;
    private final SendSuccessCallBack successCallBack;

    public DiagnoseMessageProducer(DefaultMQProducer producer, GCTRocketMQProperties properties, SendFailCallBack failCallBack, SendSuccessCallBack successCallBack) {
        this.producer = producer;
        this.properties = properties;
        this.failCallBack = failCallBack;
        this.successCallBack = successCallBack;
    }


    @Override
    public void init() {
        try {
            producer.start();
        } catch (MQClientException e) {
            log.error("producer can not start ,cause by:{}", e.getErrorMessage());
        }
    }

    @Override
    public void send(MessageBody msgBody) {
        MessageProducer.super.send(msgBody);
        if (!(msgBody instanceof DiagnoseMsg)) {
            log.error("Not supported type of msg body:{},and only supported 'DiagnoseMsg'", msgBody.getClass().getTypeName());
            return;
        }
        DiagnoseMsg actualMsg = (DiagnoseMsg) msgBody;
        Message msg = new Message(properties.getTopic(), properties.getTags(), actualMsg.getWellId(), JSONObject.toJSONString(actualMsg).getBytes(StandardCharsets.UTF_8));
        sendDefault(producer, msg, (MessageBody) actualMsg, failCallBack, successCallBack,0);
    }

    @Override
    public void destroy() {
        producer.shutdown();
    }
}
