package com.gct.tools.etlcamelhuge.MQ;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.apache.rocketmq.client.AccessChannel;
import org.apache.rocketmq.client.producer.DefaultMQProducer;
import org.apache.rocketmq.spring.support.RocketMQUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;
import springfox.documentation.spring.web.json.Json;

/**
 * class name: GCTRocketMQConfiguration
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/6/24 下午6:30
 */
@Configuration
@EnableConfigurationProperties({GCTMutableMQProperties.class})
public class GCTRocketMQConfiguration {

    public static DefaultMQProducer buildDefaultMQProducer(GCTRocketMQProperties properties) {
        GCTRocketMQProperties.Producer producerConfig = properties.getProducer();
        String nameServer = properties.getNameServer();
        String groupName = producerConfig.getGroup();
        Assert.hasText(nameServer, "[rocketmq.name-server] must not be null");
        Assert.hasText(groupName, "[rocketmq.producer.group] must not be null");
        String accessChannel = properties.getAccessChannel();
        String ak = properties.getProducer().getAccessKey();
        String sk = properties.getProducer().getSecretKey();
        boolean isEnableMsgTrace = properties.getProducer().isEnableMsgTrace();
        String customizedTraceTopic = properties.getProducer().getCustomizedTraceTopic();
        DefaultMQProducer producer = RocketMQUtil.createDefaultMQProducer(groupName, ak, sk, isEnableMsgTrace, customizedTraceTopic);
        producer.setNamesrvAddr(nameServer);
        if (!StringUtils.isEmpty(accessChannel)) {
            producer.setAccessChannel(AccessChannel.valueOf(accessChannel));
        }

        producer.setSendMsgTimeout(producerConfig.getSendMessageTimeout());
        producer.setRetryTimesWhenSendFailed(producerConfig.getRetryTimesWhenSendFailed());
        producer.setRetryTimesWhenSendAsyncFailed(producerConfig.getRetryTimesWhenSendAsyncFailed());
        producer.setMaxMessageSize(producerConfig.getMaxMessageSize());
        producer.setCompressMsgBodyOverHowmuch(producerConfig.getCompressMessageBodyThreshold());
        producer.setRetryAnotherBrokerWhenNotStoreOK(producerConfig.isRetryNextServer());
        return producer;
    }

    @Bean(name = "diagnoseMessageProducer", destroyMethod = "destroy", initMethod = "init")
    public MessageProducer diagnoseMessageProducer(GCTMutableMQProperties properties,
                                                   @Qualifier("defaultMsgSendSuccessCallBack") SendSuccessCallBack successCallBack) {
        GCTRocketMQProperties one = properties.getOne();
        return new DiagnoseMessageProducer(buildDefaultMQProducer(one), one, (e, msg) -> {
            DefaultMsgSendFailCallBack.accept(msg, one.getLogFilePath(),DiagnoseMessageProducer.class);
        }, successCallBack);
    }

    @Bean(name = "warnMessageProducer", destroyMethod = "destroy", initMethod = "init")
    public MessageProducer warnMessageProducer(GCTMutableMQProperties properties,
                                               @Qualifier("defaultMsgSendSuccessCallBack") SendSuccessCallBack successCallBack) {
        GCTRocketMQProperties two = properties.getTwo();
        return new WarnMessageProducer(buildDefaultMQProducer(two), two, (e, msg) -> {
            DefaultMsgSendFailCallBack.accept(msg, two.getLogFilePath(),WarnMessageProducer.class);
        }, successCallBack);
    }

}
