package com.gct.tools.etlcamelhuge.MQ;

import com.gct.tools.etlcamelhuge.entity.DiagnoseMsg;
import org.apache.rocketmq.client.exception.MQClientException;
import org.apache.rocketmq.client.producer.DefaultMQProducer;
import org.apache.rocketmq.client.producer.SendCallback;
import org.apache.rocketmq.client.producer.SendResult;
import org.apache.rocketmq.common.message.Message;
import org.apache.rocketmq.remoting.exception.RemotingException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Objects;

/**
 * class name: MessageProducer
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/6/24 下午6:41
 */

public interface MessageProducer {
    void init();

    default void send(MessageBody msgBody) {
        LoggerFactory.getLogger(MessageProducer.class).info("sending msg :==>{}", ((DiagnoseMsg) msgBody).getWellId() + " " + ((DiagnoseMsg) msgBody).getProdDate() + " ");
    }

    void destroy() throws Exception;

    default void sendDefault(DefaultMQProducer producer, Message msg, MessageBody actualMsg, SendFailCallBack failCallBack, SendSuccessCallBack successCallBack,int retryCount) {
        Logger log = LoggerFactory.getLogger(producer.getProducerGroup() + producer.getClass().getTypeName());
        try {
            producer.send(msg, new SendCallback() {
                @Override
                public void onSuccess(SendResult sendResult) {
                    if (Objects.nonNull(successCallBack)) successCallBack.accept(sendResult);
                }

                @Override
                public void onException(Throwable throwable) {
                    if (retryCount < 2) {
                        log.debug("retry send msg ,last time failed cause by {}",throwable.getMessage());
                        sendDefault(producer, msg, actualMsg, failCallBack, successCallBack, retryCount + 1);
                    } else {
                        if (Objects.nonNull(failCallBack)) failCallBack.accept(throwable, actualMsg);
                    }
                }
            });
        } catch (MQClientException | RemotingException | InterruptedException e) {
            log.error("Send msg failed,by cause:{},stack:{}", e.getMessage(), e.getStackTrace());
        }
    }


}
