package com.gct.tools.etlcamelhuge.camelconfig;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.aop.interceptor.AsyncUncaughtExceptionHandler;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.scheduling.annotation.AsyncConfigurer;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import java.lang.reflect.Method;

/**
 * class name: AsyncConfiguration
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/6/25 下午1:45
 */
@Configuration
@EnableAsync
public class AsyncConfiguration implements AsyncConfigurer {

    @Bean
    public AsyncTaskExecutor asyncTaskExecutor() {
        ThreadPoolTaskExecutor executor = new ThreadPoolTaskExecutor();
        executor.setCorePoolSize(8);
        executor.setMaxPoolSize(8);
        return executor;
    }
    @Override
    public AsyncUncaughtExceptionHandler getAsyncUncaughtExceptionHandler(){
        return new LogAsyncUncaughtExceptionHandler();
    }
    class LogAsyncUncaughtExceptionHandler implements  AsyncUncaughtExceptionHandler{

        @Override
        public void handleUncaughtException(Throwable ex, Method method, Object... params) {
            String paramLog="";
            int i=0;
            for (Object param : params) {
                paramLog+="param:"+i+"=>"+param.toString()+" ";
            }
            Logger log = LoggerFactory.getLogger(this.getClass());
            log.error("Method:{} run error,Exception Message:{},StackTrace:{},input params:{}",method.getName(),ex.getMessage(),ex.getStackTrace(),paramLog);
        }
    }
}
