package com.gct.tools.etlcamelhuge.routeconfig;

import lombok.extern.slf4j.Slf4j;
import org.apache.camel.Message;
import org.apache.camel.builder.RouteBuilder;
import org.apache.camel.component.file.GenericFile;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;

/**
 * class name: CamelJMSConfiguration
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/4/14 下午3:17
 */
@Slf4j
//@Configuration
public class CamelJMSConfiguration {
    @Bean
    RouteBuilder routeBuilderJms() {
        return new RouteBuilder() {
            @Override
            public void configure() throws Exception {
                from("file:{{user.home}}/Desktop/cameltest/out")
                        .routeId("file-to-jms")
                        .transform()
                        .body(GenericFile.class, gf -> {
                            final BufferedReader in;
                            try {
                                in = new BufferedReader(new InputStreamReader(new FileInputStream((File) gf.getFile())));
                                StringBuilder sb = new StringBuilder();
                                while (true) {
                                    String s = in.readLine();
                                    if (s == null || s.isEmpty()) break;
                                    sb.append(s);
                                }
                                return sb.toString();
                            } catch (Exception e) {
                                throw new RuntimeException("read fault!");
                            }
                        })
                        .process()
                        .exchange(exchange -> {
                            final Message in = exchange.getIn();
                            String body = in.getBody(String.class);
                            log.info("exchange is :" + body);
                        })
                        .to("jms:queue:testfiles").tracing();

                from("jms:queue:testfiles")
                        .routeId("jms-to-file")
                        .transform()
                        .body(String.class, s -> s + "amazing\n")
                        .process()
                        .exchange(exchange -> {
                            final Message in = exchange.getIn();
                            log.info("from jms exchange is :" + in.getBody(String.class));
                        })
                        .to("file:{{user.home}}/Desktop/cameltest/jmsout").tracing();
            }
        };
    }

    /* @Component
    static class DefaultJmsComponent implements ComponentCustomizer<JmsComponent> {

        private final ConnectionFactory connectionFactory;

        DefaultJmsComponent(ConnectionFactory connectionFactory) {
            this.connectionFactory = connectionFactory;
        }

        @Override
        public void customize(JmsComponent component) {
            component.setConnectionFactory(connectionFactory);
        }
    }*/
}
